## vim:ts=4:et:nowrap
##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
##---------------------------------------------------------------------------##


# imports
import sys

# PySol imports
if sys.modules.has_key('pysoltk'):                                  #bundle#
    from plugin import Plugin, registerGame, registerPlugin         #bundle#
    from stack import *                                             #bundle#
    from hint import Hint, DefaultHint                              #bundle#
    from util import KING, RANKS                                    #bundle#
    from pysoltk import MfxCanvasText                               #bundle#


# /***********************************************************************
# //
# ************************************************************************/

class Calculation_Hint(DefaultHint):
    # FIXME: demo logic is a complete nonsense
    def _getMoveWasteScore(self, score, color, r, t, pile, rpile):
        assert r is self.game.s.waste and len(pile) == 1
        score = 30000
        if len(t.cards) == 0:
            score = score - (KING - r.cards[0].rank) * 1000
        elif t.cards[-1].rank < r.cards[0].rank:
            score = 10000 + t.cards[-1].rank - len(t.cards)
        elif t.cards[-1].rank == r.cards[0].rank:
            score = 20000
        else:
            score = score - (t.cards[-1].rank - r.cards[0].rank) * 1000
        return score, color


# /***********************************************************************
# //
# ************************************************************************/

class Calculation_Talon(WasteTalonStack):
    def _shuffleHook(self, cards, random):
        # prepare first cards
        topcards = [ None, None, None, None ]
        for c in cards[:]:
            if c.rank <= 3 and not topcards[c.rank]:
                topcards[c.rank] = c
                cards.remove(c)
        topcards.reverse()
        return cards + topcards

    def canDealCards(self):
        return len(self.cards) > 0 and len(self.game.s.waste.cards) == 0


class Calculation_Foundation(SuitFoundationStack):
    def updateText(self):
        if self.text:
            if len(self.cards) == 0:
                rank = self.cap.base
            else:
                rank = (self.cards[-1].rank + self.cap.dir) % self.cap.mod
            self.text['text'] = RANKS[rank]

    def getBottomFace(self):
        return self._getBottomImage(self.game.app.images.getLetter(self.cap.base))


class Calculation_RowStack(BasicRowStack):
    def acceptsPile(self, from_stack, cards, stackcards=None):
        if stackcards is None: stackcards = self.cards
        if not BasicRowStack.acceptsPile(self, from_stack, cards, stackcards):
            return 0
        # this stack accepts any one card from the Waste pile
        return from_stack is self.game.s.waste and len(cards) == 1

    def getBottomFace(self):
        return self._getBottomImage(self.game.app.images.getReserveBottom())


# /***********************************************************************
# // Calculation
# ************************************************************************/

class Calculation(Plugin):

    #
    # game layout
    #

    def createGame(self):
        images = self.app.images
        XMARGIN = 10
        YMARGIN = 8
        XSPACE = images.CARDW + XMARGIN
        YSPACE = images.CARDH + YMARGIN

        # set window
        # (piles up to 20 cards are playable in default window size)
        h = max(2*YSPACE, 20*images.FACE_UP_YOFFSET)
        self.setSize(5.5*XSPACE+XMARGIN, YMARGIN + YSPACE + 30 + h)

        # create stacks
        x0 = XMARGIN + XSPACE * 3 / 2
        x = x0
        y = YMARGIN
        for i in range(4):
            s = Calculation_Foundation(x, y, self, base=i, mod=13, dir=i+1)
            self.s.foundations.append(s)
            s.text = MfxCanvasText(self.canvas,
                                       x + images.CARDW / 2, y + YSPACE,
                                       anchor='n')
            s.text['font'] = self.getLargeFont()
            x = x + XSPACE
        self.info_text = MfxCanvasText(self.canvas,
                                           x + 2 * XMARGIN, y + images.CARDH / 2,
                                           anchor='w')
        self.info_text['text'] = (
            "1:  A 2 3 4 5 6 7 8 9 T J Q K\n" +
            "2:  2 4 6 8 T Q A 3 5 7 9 J K\n" +
            "3:  3 6 9 Q 2 5 8 J A 4 7 T K\n" +
            "4:  4 8 Q 3 7 J 2 6 T A 5 9 K")
        x = x0
        y = YMARGIN + YSPACE + 30
        for i in range(4):
            self.s.rows.append(Calculation_RowStack(x, y, self, max_accept=1))
            x = x + XSPACE
        self.setRegion(self.s.rows, [ x0 - XSPACE / 2, y, 999999, 999999 ])
        x = XMARGIN
        self.s.talon = Calculation_Talon(x, y, self, max_rounds=1)
        self.s.talon.text = MfxCanvasText(self.canvas,
                               x + images.CARDW / 2, y - YMARGIN,
                               anchor='s')
        self.s.talon.text_format = "%d"
        y = y + YSPACE
        self.s.waste = WasteStack(x, y, self)

        # create cards
        self.cards = self.s.talon.createCards(1)

        # define stack-groups
        self.sg.openstacks = self.s.foundations + self.s.rows
        self.sg.talonstacks = [self.s.talon] + [self.s.waste]
        self.sg.dropstacks = self.s.rows + [self.s.waste]

    #
    # game overrides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 1 * 52
        self.moves.state = self.S_DEAL
        self.s.talon.dealRow(rows=self.s.foundations)
        self.s.talon.dealCards()          # deal first card to WasteStack

    def fillStack(self, stack):
        if 1 and stack is self.s.waste:
            # convenience - automatically deal a card
            if self.canDealCards():
                self.dealCards()

    def getHighlightPilesStacks(self):
        return None

    def getHintClass(self):
        return Calculation_Hint


# register the plugin: id, class, name, version and minimum PySol version
registerPlugin(256, Calculation, "Calculation", "1.01", "2.11")

