## vim:ts=4:et:nowrap
##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Andrew Csillag <drew_csillag@geocities.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
##---------------------------------------------------------------------------##


# imports
import sys

# PySol imports
if sys.modules.has_key('pysoltk'):                                  #bundle#
    from plugin import Plugin, registerGame, registerPlugin         #bundle#
    from stack import *                                             #bundle#
    from game import Game                                           #bundle#
    from hint import Hint, DefaultHint                              #bundle#
    from util import RANKS                                          #bundle#
    from pysoltk import MfxCanvasText                               #bundle#


# /***********************************************************************
# // Canfield
# ************************************************************************/

class Canfield(Plugin):

    #
    # game layout
    #

    def createGame(self):
        images = self.app.images
        XMARGIN = 10
        YMARGIN = 10
        XSPACE = images.CARDW + XMARGIN
        YSPACE = images.CARDH + YMARGIN

        # set window
        # (piles up to 20 cards are playable in default window size)
        h = max(3*YSPACE, 20*images.FACE_UP_YOFFSET)
        self.setSize(6*XSPACE+2*XMARGIN, YMARGIN + YSPACE + 20 + h)

        # extra settings
        self.base_card = None

        # create stacks
        x = XMARGIN
        y = YMARGIN
        self.s.talon = WasteTalonStack(x, y, self, num_deal=3)
        self.s.talon.text = MfxCanvasText(self.canvas,
                                              x + images.CARDW / 2,
                                              y + YSPACE,
                                              anchor='center')
        self.s.talon.text_format = "%d"
        x = x + XSPACE
        self.s.waste = WasteStack(x, y, self)
        self.s.waste.text = MfxCanvasText(self.canvas,
                                              x + images.CARDW / 2,
                                              y + YSPACE,
                                              anchor='center')
        self.s.waste.text_format = "%d"
        x = x + XMARGIN
        y = YMARGIN
        for i in range(4):
            x = x + XSPACE
            self.s.foundations.append(SuitFoundationStack(x, y, self, i, mod=13))
        self.info_text = MfxCanvasText(self.canvas,
                               x + XSPACE + XMARGIN, y + images.CARDH,
                               anchor='sw')
        self.info_text['font'] = self.getLargeFont()
        x = XMARGIN
        y = YMARGIN + YSPACE + 20
        self.s.reserves.append(OpenStack(x, y, self))
        if 1:
            # this only makes sense if the reserve is face-down
            self.s.reserves[0].FACE_UP_YOFFSET = 10
            self.s.reserves[0].FACE_DOWN_YOFFSET = 10
        else:
            self.s.reserves[0].text = MfxCanvasText(self.canvas,
                                                 x + images.CARDW / 2,
                                                 y + YSPACE,
                                                 anchor='center')
            self.s.reserves[0].text_format = "%d"
        x = XMARGIN + 2 * XSPACE + XMARGIN
        for i in range(4):
            self.s.rows.append(RowStack(x, y, self, mod=13))
            x = x + XSPACE

        # create cards
        self.cards = self.s.talon.createCards(1)

        # define stack-groups
        self.sg.openstacks = self.s.foundations + self.s.rows
        self.sg.talonstacks = [self.s.talon] + [self.s.waste]
        self.sg.dropstacks = self.s.rows + [self.s.waste] + self.s.reserves
        self.sg.reservestacks = self.s.reserves

    #
    # game extras
    #

    def updateText(self):
        if not self.base_card:
            t = ""
        else:
            t = RANKS[self.base_card.rank]
        self.info_text['text'] = t

    #
    # game overrides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 1 * 52
        self.moves.state = self.S_DEAL
        self.base_card = None
        self.updateText()
        # deal base_card to foundations, update foundations cap.base
        self.base_card = self.s.talon.getCard()
        for s in self.s.foundations:
            s.cap.base = self.base_card.rank
        to_stack = self.s.foundations[self.base_card.suit]
        self.flipMove(self.s.talon)
        self.moveMove(1, self.s.talon, to_stack)
        self.updateText()
        # fill the stock
        for i in range(13):
            self.moveMove(1, self.s.talon, self.s.reserves[0], frames=4, shadow=0)
        self.flipMove(self.s.reserves[0])
        self.s.talon.dealRow(reverse=1)
        self.s.talon.dealCards()          # deal first 3 cards to WasteStack

    def fillStack(self, stack):
        if stack in self.s.rows:
            if len(stack.cards) == 0 and len(self.s.reserves[0].cards) > 0:
                self.moveMove(1, self.s.reserves[0], stack, frames=-1)
                self.fillStack(self.s.reserves[0])
        elif stack is self.s.waste or stack in self.s.reserves:
            # make sure the top card of the Waste/Stock is face up
            if len(stack.cards) > 0 and not stack.cards[-1].face_up:
                self.flipMove(stack)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.color != card2.color and
                ((card1.rank + 1) % 13 == card2.rank or (card2.rank + 1) % 13 == card1.rank))

    def getHintClass(self):
        return DefaultHint

    def _restoreGameHook(self, game):
        self.base_card = self.cards[game.loadinfo.base_card_id]
        for s in self.s.foundations:
            s.cap.base = self.base_card.rank
        self.updateText()

    def _loadGameHook(self, p):
        self.loadinfo.addattr(base_card_id=None)    # register extra load var.
        self.loadinfo.base_card_id = p.load()

    def _saveGameHook(self, p):
        p.dump(self.base_card.id)


registerPlugin(262, Canfield, "Canfield", "1.01", "2.11")

