## vim:ts=4:et:nowrap
##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
##---------------------------------------------------------------------------##


# imports
import sys

# PySol imports
if sys.modules.has_key('pysoltk'):                                  #bundle#
    from plugin import Plugin, registerGame, registerPlugin         #bundle#
    from stack import *                                             #bundle#
    from game import Game                                           #bundle#
    from hint import Hint, DefaultHint, CautiousDefaultHint         #bundle#
    from util import KING                                           #bundle#
    from pysoltk import MfxCanvasText                               #bundle#


# /***********************************************************************
# //
# ************************************************************************/

class Golf_RowStack(BasicRowStack):
    def clickHandler(self, event):
        if self.canDropCard(self.game.s.foundations):
            return self.rightclickHandler(event)
        return 0


class Golf_WasteStack(WasteStack):
    def canMovePile(self, cards):
        return 0

    def acceptsPile(self, from_stack, cards, stackcards=None):
        if stackcards is None: stackcards = self.cards
        if not self.basicAcceptsPile(from_stack, cards, stackcards):
            return 0
        # check cards
        assert len(cards) == 1 and len(stackcards) >= 1
        r1, r2 = stackcards[-1].rank, cards[0].rank
        mod = 13
        if self.game.getStrictness() > 0:
            # nothing on a King, no wrap-around
            if r1 == KING:
                return 0
            mod = 1024
        return (r1 + 1) % mod == r2 or (r2 + 1) % mod == r1


# /***********************************************************************
# // Golf
# ************************************************************************/

class Golf(Plugin):

    #
    # game layout
    #

    def createGame(self):
        images = self.app.images
        XMARGIN = 10
        YMARGIN = 8
        XSPACE = images.CARDW + XMARGIN
        YSPACE = images.CARDH + YMARGIN
        XOFFSET = 10

        # set window
        w1, w2 = 8*XSPACE+XMARGIN, 2*XSPACE
        if w2 + 52*XOFFSET > w1:
            XOFFSET = int((w1 - w2) / 52)
        self.setSize(w1, 4*YSPACE+YMARGIN)

        # create stacks
        x = XMARGIN + XSPACE / 2
        y = YMARGIN
        for i in range(7):
            self.s.rows.append(Golf_RowStack(x, y, self))
            x = x + XSPACE
        x = XMARGIN
        y = self.height - YSPACE
        self.s.talon = WasteTalonStack(x, y, self, max_rounds=1)
        self.s.talon.text = MfxCanvasText(self.canvas,
                               x + images.CARDW / 2, y - YMARGIN,
                               anchor='s')
        self.s.talon.text_format = "%d"
        x = x + XSPACE
        self.s.waste = Golf_WasteStack(x, y, self, max_accept=1)
        self.s.waste.FACE_UP_XOFFSET = [ XOFFSET ]
        self.s.waste.text = MfxCanvasText(self.canvas,
                               x + images.CARDW / 2, y - YMARGIN,
                               anchor='s')
        self.s.waste.text_format = "%d"
        # the Waste is also our only Foundation in this game
        self.s.foundations.append(self.s.waste)

        # create cards
        self.cards = self.s.talon.createCards(1)

        # define stack-groups
        self.sg.openstacks = [self.s.waste]
        self.sg.talonstacks = [self.s.talon]
        self.sg.dropstacks = self.s.rows

    #
    # game overrides
    #

    def getStrictness(self):
        return 1

    def startGame(self):
        assert len(self.s.talon.cards) == 1 * 52
        self.moves.state = self.S_DEAL
        for i in range(4):
            self.s.talon.dealRow(frames=0)
        self.s.talon.dealRow()
        self.s.talon.dealCards()          # deal first card to WasteStack

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        if card1.rank == KING:
            return 0
        return (card1.rank + 1 == card2.rank or card2.rank + 1 == card1.rank)

    def getHighlightPilesStacks(self):
        return None

    def getAutoDropStacks(self):
        return None

    def getHintClass(self):
        return DefaultHint


# /***********************************************************************
# //
# ************************************************************************/

class RelaxedGolf(Golf):
    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return ((card1.rank + 1) % 13 == card2.rank or (card2.rank + 1) % 13 == card1.rank)

    def getStrictness(self):
        return 0


# register the plugin: id, class, name, version and minimum PySol version
registerPlugin(259, Golf, "Golf", "1.00", "2.11")
registerPlugin(260, RelaxedGolf, "Golf [relaxed]", "1.00", "2.11")

