## vim:ts=4:et:nowrap
##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
##---------------------------------------------------------------------------##


# imports
import sys

# PySol imports
if sys.modules.has_key('pysoltk'):                                  #bundle#
    from plugin import Plugin, registerGame, registerPlugin         #bundle#
    from stack import *                                             #bundle#
    from game import Game                                           #bundle#
    from hint import Hint, DefaultHint, CautiousDefaultHint         #bundle#
    from pysoltk import MfxCanvasText                               #bundle#


# /***********************************************************************
# //
# ************************************************************************/

class GrandfathersClock_Hint(CautiousDefaultHint):
    # FIXME: demo is not too clever in this game

    def _getDropCardScore(self, score, color, r, t, pile, rpile):
        # drop all cards immediately
        return 92000, color


# /***********************************************************************
# //
# ************************************************************************/

class GrandfathersClock_Talon(InvisibleTalonStack):
    def _shuffleHook(self, cards, random):
        # move clock cards to bottom of the Talon (i.e. last cards to be dealt)
        C, S, H, D = 0*13, 1*13, 2*13, 3*13
        ids = (1+S, 2+H, 3+C, 4+D, 5+S, 6+H, 7+C, 8+D, 9+S, 10+H, 11+C, 12+D)
        clocks = []
        for c in cards[:]:
            if c.id in ids:
                clocks.append(c)
                cards.remove(c)
        # sort clocks reverse by rank
        clocks.sort(lambda a, b: cmp(b.rank, a.rank))
        return clocks + cards


class GrandfathersClock_RowStack(SequenceRowStack):
    def isSequence(self, cards):
        return isRankSequence(cards, self.cap.mod, self.cap.dir)


# /***********************************************************************
# // Grandfather's Clock
# ************************************************************************/

class GrandfathersClock(Plugin):

    #
    # game layout
    #

    def createGame(self):
        images = self.app.images
        XMARGIN = 10
        YMARGIN = 8
        XSPACE = images.CARDW + XMARGIN
        YSPACE = images.CARDH + YMARGIN
        XOFFSET = 10

        # set window
        # (piles up to 9 cards are fully playable in default window size)
        dh = max(3*YSPACE/2+images.CARDH, YSPACE+(9-1)*images.FACE_UP_YOFFSET)
        self.setSize(10*XSPACE+XMARGIN, YMARGIN+2*dh)

        # create stacks
        for j in range(2):
            x, y = XMARGIN, YMARGIN + j*dh
            for i in range(4):
                self.s.rows.append(GrandfathersClock_RowStack(x, y, self, max_accept=1))
                x = x + XSPACE
        y = YMARGIN + dh - images.CARDH / 2
        self.setRegion(self.s.rows[:4], [ -999, -999, x - XMARGIN / 2, y ])
        self.setRegion(self.s.rows[4:], [ -999,    y, x - XMARGIN / 2, 999999 ])
        d = [ (0,0), (1,0.15), (2,0.5), (2.5,1.5), (2,2.5), (1,2.85) ]
        for i in range(len(d)):
            d.append( (0 - d[i][0], 3 - d[i][1]) )
        x0, y0 = XMARGIN, YMARGIN + dh - images.CARDH
        for i in range(12):
            j = (i + 5) % 12
            x = int(round(x0 + ( 6.5+d[j][0]) * XSPACE))
            y = int(round(y0 + (-1.5+d[j][1]) * YSPACE))
            suit = (1, 2, 0, 3) [i % 4]
            self.s.foundations.append(SuitFoundationStack(x, y, self, suit,
                                                          base=i+1, mod=13,
                                                          max_move=0))
        self.s.talon = GrandfathersClock_Talon(self.width-XSPACE, self.height-YSPACE, self)

        # create cards
        self.cards = self.s.talon.createCards(1)

        # define stack-groups
        self.sg.openstacks = self.s.foundations + self.s.rows
        self.sg.talonstacks = [self.s.talon]
        self.sg.dropstacks = self.s.rows

    #
    # game overrides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 1 * 52
        self.moves.state = self.S_DEAL
        for i in range(4):
            self.s.talon.dealRow(frames=0)
        self.s.talon.dealRow()
        self.s.talon.dealRow(rows=self.s.foundations)
        assert len(self.s.talon.cards) == 0

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.suit == card2.suit and
                (card1.rank + 1 == card2.rank or card2.rank + 1 == card1.rank))

    def getHighlightPilesStacks(self):
        return None

    def getAutoDropStacks(self):
        # disable auto drop - this would ruin the whole gameplay
        return None

    def getHintClass(self):
        return GrandfathersClock_Hint


# register the plugin: id, class, name, version and minimum PySol version
registerPlugin(261, GrandfathersClock, "Grandfather's Clock", "1.00", "2.11")

